#!/bin/bash
#
# Name: element-admin.sh
# Date: 2012-08-23
# Author: Marjan Jemec <jemec@iskratel.si>
#
# API for element management
#

###################################################
###################################################
# Begin of Common variables

PROGRAM_VERSION=2.8.6
PROGRAM_NAME=${0##*/}

export PATH=${PATH}:/usr/local/sbin:/usr/sbin:/sbin

################################################################################
# Debug mode setup

if [ "x${DEBUG_MODE}" = "x" ]
then
	DEBUG_MODE=false
	#DEBUG_MODE=true
fi

DEBUG_LOGFILE=${HOME}/$(basename $0)-$(date +%y)$(date +%m)$(date +%d)_$(date +%H)$(date +%M)$(date +%S).log

if [ "x${DEBUG_MODE}" = "xtrue" ]
then
	logfile=${DEBUG_LOGFILE}
	mkfifo ${logfile}.pipe
	tee < ${logfile}.pipe ${logfile} &
	exec &> ${logfile}.pipe
	rm ${logfile}.pipe

	set -x
fi
################################################################################

APP_ROOT_DIR=/opt/si2000/neroot

DB_SERVER_SYSTEM_DEFAULT=solid

################################################################################
#
# Program locations
#

SYSTEM_BOARD_INFO_CMD=/sbin/system-board-info

# Define common HA global variables
HA_SW_TOOL_CMD=/sbin/ha-sw-tool
HA_BOOT_CMD=/sbin/ha-boot
HA_COMMON_CFG=/etc/ha.d/ha-functions

DB_FUNCTIONS_COMMON_FILE=/sbin/db-functions
ELEMENT_ADMIN_FUNCTIONS_FILE=/sbin/element-functions

# Define variables for alarming backup package
BOOT_LATE_ACTIONS_DIR=/etc/actions/boot-late

CMD_BOOT_ACTION=${BOOT_LATE_ACTIONS_DIR}/report-backup-package
CMD_RETREAT=/sbin/itRetreatBSPR
CMD_REPORT=/sbin/itReportBSPR

RPM_DB_TEST_CMD=/sbin/system-rpm-database-repair

# Temporary environment file for saving parsed command line switches
TMP_ENV=/tmp/$(basename $0).env.$$

################################################################################
#
# Misc variables
#

# Dababase server read/write modes
DB_READ_WRITE="RW"
DB_READ_ONLY="RO"

# Load common HA global variables
if [ -f ${HA_COMMON_CFG} ]
then
	. ${HA_COMMON_CFG}
else
	echo "Error: missing ${HA_COMMON_CFG}"
	exit 2
fi

# Load common DB functions and variables
if [ -f ${DB_FUNCTIONS_COMMON_FILE} ]
then
	. ${DB_FUNCTIONS_COMMON_FILE}
else
	echo "Error: missing ${DB_FUNCTIONS_COMMON_FILE}"
	exit 2
fi

# Load common element-admin variables
if [ -f ${ELEMENT_ADMIN_FUNCTIONS_FILE} ]
then
	. ${ELEMENT_ADMIN_FUNCTIONS_FILE}
else
	echo "Error: missing ${ELEMENT_ADMIN_FUNCTIONS_FILE}"
	exit 2
fi

# Check if RPM database is working
test -f ${RPM_DB_TEST_CMD} && ${RPM_DB_TEST_CMD}

# End of Common variables
################################################################################
################################################################################


################################################################################
################################################################################
# General functions
################################################################################

################################################################################
# Help function
function show_help()
{
	cat << EOF

Usage:
	${0##*/} <command> [args]

General commands
	--help  - show this help

	--get-api-version  - read API script version

System management commands
	--sys-info  - print collected system information
		Additional arguments
			[--no-ssh] - do not execute commands via ssh

	--list-pkg  - show installed products
		Additional arguments
			[--no-ssh] - do not execute commands via ssh

	--test-running-installation - test if installation is already running

	--install  - install defined product to the reserved partitions
		Additional arguments:
			--nodeid=<nodeID>
			--datarelease=<DATARELEASE>
			--data-type=<data type>
			--housing=<HOUSING>
			--repo-sw-url=<SW_URL>
			[--target-board-name=<BOARD NAME>]
			[--use-api-version=<VERSION>]
			[--repo-fs-url=<FS_URL>]
			[--target-arch=<arch>]
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--install-filesystem-only  - install defined filesystem only to the reserved partitions
		Additional arguments:
			--repo-sw-url=<SW_URL>
			[--repo-fs-url=<FS_URL>]
			[--target-arch=<arch>]
	
	--install-platform-only  - install defined system platform only to the reserved partitions
		Additional arguments:
			--repo-sw-url=<SW_URL>
			[--repo-fs-url=<FS_URL>]
			[--target-arch=<arch>]
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--install-on-working  - install defined product to the working partitions
		Additional arguments:
			--nodeid=<nodeID>
			--datarelease=<DATARELEASE>
			--data-type=<data type>
			--housing=<HOUSING>
			--repo-sw-url=<SW_URL>
			[--target-board-name=<BOARD NAME>]
			[--use-api-version=<VERSION>]
			[--repo-fs-url=<FS_URL>]
			[--target-arch=<arch>]
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--set-active-release  - activate specified product version
		Additional arguments
			--product=[WORKING|RESERVED] - define which OS is default to boot
			[--no-ssh] - do not execute commands via ssh

Database management commands
	--get-solid-uid  - print solid connection parameters

	--show-db-status - print database server information

	--activate-replication  - start replicate data between master node and network element
		Additional arguments:
			--use-api-version=<VERSION> 
			--nodeid=<nodeID>
			--dbrelease=<DBRELEASE>
			--datarelease=<DATARELEASE> 
			--master-node-name=<MASTER_NODE_NAME>
			--replica-node-name=<REPLICA_NODE_NAME>
			--master-connection-string=<MASTER_CONNECTION_STRING>
			--master-username=<MASTER_USERNAME> 
			--master-password=<MASTER_PASSWORD>
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--deactivate-replication  - stop replicate data between master node and network element
			--use-api-version=<VERSION> 
			[--nodeid=<nodeID>]
			--dbrelease=<DBRELEASE>
			--datarelease=<DATARELEASE> 
			--master-node-name=<MASTER_NODE_NAME>
			--replica-node-name=<REPLICA_NODE_NAME>
			--master-connection-string=<MASTER_CONNECTION_STRING>
			--master-username=<MASTER_USERNAME> 
			--master-password=<MASTER_PASSWORD>
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--refresh  - refresh database
			--use-api-version=<VERSION>
			--nodeid=<nodeID>
			--dbrelease=<DBRELEASE>
			--datarelease=<DATARELEASE>
			[--catalog-name=<CATALOG>] 
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--upgrade  - run activate replication, refresh, and deactivate replication of datbase.
			--use-api-version=<VERSION> 
			--nodeid=<nodeID>
			--dbrelease=<DBRELEASE>
			--datarelease=<DATARELEASE> 
			--master-node-name=<MASTER_NODE_NAME>
			--replica-node-name=<REPLICA_NODE_NAME>
			--master-connection-string=<MASTER_CONNECTION_STRING>
			--master-username=<MASTER_USERNAME> 
			--master-password=<MASTER_PASSWORD>
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--install-data-from-disc  - load data into database; control variables are saved in /etc/sysconfig/db-data
		Additional arguments:
			--product=[WORKING|RESERVED] - define which OS has data to load
			[--db-server=[postgresql|netezza|solid]]  - default DB server: solid

	--without-db-data  - installing a product without database and DB data

	--without-load-db-data  - do not load DB data from disc into database after installing of product

EOF
}


################################################################################
################################################################################
# System management functions
################################################################################

function do_check_if_installation_in_progress()
{
	# Check if installation on this node is running
	# Return codes:
	#   0 - installation not running
	#   1 - installation is in progress

	ps_list=$(ps ax)
	if [ $(echo "${ps_list}" | grep ${0} | grep '\-\-install' | grep -v " *$$ " | grep -v " *${PPID}" | grep -v ssh | grep -v sudo | wc -l) -gt 0 ]
	then
		return 1
	else
		return 0
	fi
}

################################################################################
# sys_info function
function do_sys_info()
{
# ./element-admin.sh --sys-info
	USER_NAME=`id --user --name`
	if [ ! "x${USER_NAME}" = "xroot" ]
	then
		SUDO_CMD=sudo
	else
		unset SUDO_CMD
	fi

	msg -d "START: do_sys_info()."

	echo "Date: `date "${DATE_FORMAT}"`"

	echo ""
	echo "Board Manufacturer:  $(${SUDO_CMD} ${SYSTEM_BOARD_INFO_CMD} --manufacturer)"
	echo "Board Name:          $(${SUDO_CMD} ${SYSTEM_BOARD_INFO_CMD} --product-name)"
	echo "Board Serial number: $(${SUDO_CMD} ${SYSTEM_BOARD_INFO_CMD} --serial-number)"
	echo ""

	# Print out node state
	mwd_get_node_state
	echo -n "Node state: "
	case ${NODE_STATE} in
		${EV_NODE_ST_ACT})
			echo "ACTIVE."
			;;
		${EV_NODE_ST_SB})
			echo "STANDBY."
			;;
		${EV_NODE_ST_UNDEF})
			echo "UNDEFINED."
			;;
		${EV_HAM_NOT_RESPONDING})
			echo "Cannot get node state. HAM is not responding."
			;;
		*)
			echo ${NODE_STATE}
			;;
	esac

	echo ""

	cmd_uptime=`which uptime`
	if [ ! "x${cmd_uptime}" = "x" ]
	then
		echo -n "Uptime: "
		${cmd_uptime} | sed 's/  / /g' | cut -d' ' -f4-
	fi

	cmd_free=`which free`
	if [ ! "x${cmd_free}" = "x" ]
	then
		echo "Memory information (total, used, free) in kB:"
		${cmd_free} -t -k | awk '{if($1 ~ /Mem:|Swap:|Total:/){print "  " $1 " " $2 " " $3 " " $4;}}'
	fi

	# Show disk info.
	echo ""
	echo "Filesystem information (1M blocks):"
	df -mP 2>/dev/null

	# Print out symbolic links
	echo ""
	echo "Symbolic links on working:"
	sym_link=`readlink ${APP_ROOT_DIR}/active`
	if [ "x${sym_link}" = "x" ]
	then
		sym_link="-"
	fi
	echo "  active.: ${sym_link}"

	sym_link=`readlink ${APP_ROOT_DIR}/current`
	if [ "x${sym_link}" = "x" ]
	then
		sym_link="-"
	fi
	echo "  current: ${sym_link}"

	##############################################################################
	# Fail link should be removed, look request REQ01053569
	##############################################################################  
	sym_link=`readlink ${APP_ROOT_DIR}/fail`
	if [ ! "x${sym_link}" = "x" ]
	then
		echo "  fail...: ${sym_link}"
	fi

	if [ -d ${RESERVED_OS_MOUNT_PREFIX} ]
	then
		echo ""
		echo "Symbolic links on reserved:"
		sym_link=`readlink ${RESERVED_OS_MOUNT_PREFIX}${APP_ROOT_DIR}/active`
		if [ "x${sym_link}" = "x" ]
		then
			sym_link="-"
		fi
		echo "  active.: ${sym_link}"     
		
		sym_link=`readlink ${RESERVED_OS_MOUNT_PREFIX}${APP_ROOT_DIR}/current`
		if [ "x${sym_link}" = "x" ]
		then
			sym_link="-"
		fi
		echo "  current: $sym_link"     

		##############################################################################
		# Fail link should be removed, look request REQ01053569
		##############################################################################  
		sym_link=`readlink ${RESERVED_OS_MOUNT_PREFIX}${APP_ROOT_DIR}/fail`
		if [ ! "x${sym_link}" = "x" ]
		then
			echo "  fail...: ${sym_link}"
		fi
	fi

	# Show installed products
	echo ""
	echo "Installed products:"
	${HA_BOOT_CMD} --show-product-detailed-version-all --host=${HOST_STRING}

	# Get DB state and loaded catalogs
	db_server_and_data_status

	# Print out application state
	if [ "${NODE_STATE}" = "${EV_HAM_NOT_RESPONDING}" ]
	then
		echo "Application status: Cannot get application status. HAM is not responding."
	else
		mwd_get_app_state
		echo -n "Application status: "
		case ${APP_STATE} in
			${EV_APP_ST_RUNNING})
				echo "RUNNING."
				;;
			${EV_APP_ST_STOPPED})
				echo "STOPPED."
				;;
			*)
				echo "UNKNOWN."
				;;
		esac
	fi

		##############################################################################
		# drbd overview info
		##############################################################################
	echo ""
	echo "DRBD OVERVIEW"
	drbd-overview

	msg -d "END: do_sys_info()."
	return 0
}


################################################################################
# list_pkg function
function do_list_pkg()
{
# ./element-admin.sh --list-pkg

	msg -d "START: do_list_pkg()."
	# List installed products
	${HA_BOOT_CMD} --show-product-detailed-version-all --host=${HOST_STRING}

	msg -d "END: do_list_pkg()."
	return 0
}

################################################################################
# Check if HTTPS is used
function do_handle_ssl_cert()
{
	mns_short_name=`echo ${REPO_SW_URL} | cut -d '/' -f3 | cut -d '.' -f1`

	#check if hash already created
	if [ ! -f /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt ]
	then
		#copy mns ssl cert from /opt/si2000/neroot
		#check if mns ssl cert exists on neadmin home dir
		if [ -f /opt/si2000/neroot/mns-${mns_short_name}.crt ]
		then
			mkdir -p /opt/si2000/mnroot/ssl
			mkdir -p /opt/si2000/mnroot/ssl/certs

			cp /opt/si2000/neroot/mns-${mns_short_name}.crt /opt/si2000/mnroot/ssl/certs/
			chmod 755 /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt
		else
			return 1
		fi
	fi

	#check if hash file exists on /etc/ssl/certs
	ssl_cert_hash=`openssl x509 -noout -hash -in /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt`
	if [ ! -f /etc/ssl/certs/${ssl_cert_hash}.0 ]
	then
		#echo "SSL -> Creating hash link: /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt /etc/ssl/certs/${ssl_cert_hash}.0"

		#create hash link to mns ssl cert on mnroot dir
		ln -s /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt /etc/ssl/certs/${ssl_cert_hash}.0
	fi

	return 0
}

################################################################################
# Install sw to the reserved disk space
function do_install_software()
{
	if [[ "${REPO_SW_URL}" == *"https"* ]]
	then
		# https will be used, handle ssl cert
		do_handle_ssl_cert
		ret=$?
		if [ ${ret} -ne 0 ]
		then
			echo "Error handling MNS SSL cert! - ${ret}"
			msg -e "Error handling MNS SSL cert! - ${ret}"

			do_exit ${ret}
		fi
	fi

	# Create fresh environment
	rm -rf ${RESERVED_OS_INSTALL_CFG_DIR}
	mkdir -p ${RESERVED_OS_INSTALL_CFG_DIR}
	mv ${TMP_ENV} ${RESERVED_OS_INSTALL_ENV}
	msg -d "START: do_install_software()."

	/sbin/system-install install
	rv=$? 

	msg -d "END: do_install_software() [rv=${rv}]."

        # delete SSL certs if https used
        if [[ "${REPO_SW_URL}" == *"https"* ]]
        then
                mns_short_name=`echo ${REPO_SW_URL} | cut -d '/' -f3 | cut -d '.' -f1`
                ssl_cert_hash=`openssl x509 -noout -hash -in /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt`

                if [ -f /etc/ssl/certs/${ssl_cert_hash}.0 ]
                then
                        rm /etc/ssl/certs/${ssl_cert_hash}.0
                else
                        LOG_MSG="Deleting SSL certs: /etc/ssl/certs/${ssl_cert_hash}.0 does not exist"
                        echo $LOG_MSG
                        msg -e "${LOG_MSG}"
                fi

                if [ -f /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt ]
                then
                        rm /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt
                else
                        LOG_MSG="Deleting SSL certs: /opt/si2000/mnroot/ssl/certs/mns-${mns_short_name}.crt does not exist"
                        echo $LOG_MSG
                        msg -e "${LOG_MSG}"
                fi

                if [ -f /opt/si2000/neroot/mns-${mns_short_name}.crt ]
                then
                        rm /opt/si2000/neroot/mns-${mns_short_name}.crt
                else
                        LOG_MSG="Deleting SSL certs: /opt/si2000/neroot/mns-${mns_short_name}.crt does not exist"
                        echo $LOG_MSG
                        msg -e "${LOG_MSG}"
                fi
        fi

	return ${rv}
}


################################################################################
# set_active_release function
# Prepare symbolic link and correct boot loader.
# 
function do_set_active_release()
{
	msg -i "CMD: ${PROGRAM_NAME} --set-active-release [with relevant options: --product=${PRODUCT}]"

	msg -d "START: do_set_active_release()."

	# Check needed input args
	arg_check "--product" "${PRODUCT}"

	product_side=`echo ${PRODUCT} | tr "[:upper:]" "[:lower:]" | sed "s%:.*%%g"`

	case ${product_side} in
		working*)
			use_partition=working
			use_prefix=/
			;;
		reserved*)
			use_partition=reserved
			use_prefix=${RESERVED_OS_MOUNT_PREFIX}
			;;
		*)
			msg -e "Unknown definition ${PRODUCT} from argument --product"
			return ${NEI_ERR_WRONG_ARGS}
			;;
	esac

	# Change the reporting "backup package is running" if set environment variable
	# ALARM_BACKUP_PACKAGE_SETUP=true
	if [ "x${ALARM_BACKUP_PACKAGE_SETUP}" = "xtrue" ]
	then
		# Do not mark reserved partition as backup if already running as backup package
		if [ ! -e ${CMD_BOOT_ACTION} ]
		then
			# Mark reserved partition as backup package
			# if ${CMD_REPORT} is present there
			if [ -e ${RESERVED_OS_MOUNT_PREFIX}${CMD_REPORT} ]
			then
				mkdir -p ${RESERVED_OS_MOUNT_PREFIX}${BOOT_LATE_ACTIONS_DIR}
				rm -f ${RESERVED_OS_MOUNT_PREFIX}${CMD_BOOT_ACTION}
				ln -s ${CMD_REPORT} ${RESERVED_OS_MOUNT_PREFIX}${CMD_BOOT_ACTION}
				rm -f ${CMD_BOOT_ACTION}
			fi
		else
			# Cannot switch from partition with running backup package
			msg -e "Cannot switch back from partition with currently running backup package."
			return ${NEI_ERR}
		fi
	fi

	msg -d "Activating product on ${use_partition} filesystem."

	case ${product_side} in
		working*)
			# Set boot loader to reserved partition
			${HA_BOOT_CMD} --host=${HOST_STRING} --set-working
			rv=$?
			;;

		reserved*)
			# Set boot loader to reserved partition
			${HA_BOOT_CMD} --host=${HOST_STRING} --set-reserved
			rv=$?
			;;
	esac

	# Flush changes to the hard drive
	sync

	if [ ${rv} -ne 0 ]
	then
		msg -e "Activation of product loader to ${product_side} failed."
		return ${rv}
	fi

	# Make sure to wake up into runlevel 3 after reboot
	if [ -e ${use_prefix}/etc/inittab ]
	then
		sed -i "s|id:.*initdefault:|id:3:initdefault:|" ${use_prefix}/etc/inittab
	fi
	
	# Set default runlevel for systemd
	if type systemctl >/dev/null 2>/dev/null
	then
		systemctl set-default multi-user.target
	fi

	# Test if this is not called during boot - below 60 seconds uptime.
	# It shouldn't reset flags and modify product links while starting bootcycle actions.
	if [ $(awk '{print int($1%3600)}' /proc/uptime) -gt 60 ]
	then
		# Remove any bootcycle detection flags to start reset count from beginning
		bootcycle_pids=`pgrep bootcycle-wait`
		if [ ! "x${bootcycle_pids}" = "x" ]
		then
			# Prevent to write counter result into file later.
			kill -9 "${bootcycle_pids}"
		fi

		# Create symbolic link "current" for new product
		product_dir_name=`${HA_SW_TOOL_CMD} --os=${use_partition} --host=${HOST_STRING} --get-installed-product-dir`
		if [ ! "x${product_dir_name}" = "x" ]
		then
			if [ -e ${use_prefix}/etc/init.d/it_product ]
			then
				chroot ${use_prefix} /etc/init.d/it_product install "${product_dir_name}"
				rv=$?
				if [ ${rv} -ne 0 ]
				then
					msg -e "Activation of product ${product_dir_name} failed."
					return ${rv}
				fi
			fi
		fi
	fi

	msg -i "Changing of bootloader to '${product_side}' finished [rv=${rv}]."
	msg -d "END: do_set_active_release()."

	# Flush all changes to the hard drive - final step
	sync

	return ${rv}
}


################################################################################
################################################################################
# Database management functions
################################################################################

################################################################################
# Show Solid DB uid parameters
function do_get_solid_uid()
{
	db_get_db_uid solid
	return $?
}

################################################################################
# Save replication connection parameters into ${PRODUCT_DB_REPLICA_DATA_CFG}
function do_save_replica_info()
{
	mkdir -p $(dirname ${PRODUCT_DB_REPLICA_DATA_CFG})
	cat > ${PRODUCT_DB_REPLICA_DATA_CFG} << EOF
# This file was generated by ${0##*/}

REPLICA_NODE_NAME="${REPLICA_NODE_NAME}"
MASTER_NODE_NAME="${MASTER_NODE_NAME}"
MASTER_CONNECTION_STRING="${MASTER_CONNECTION_STRING}"
MASTER_USERNAME="${MASTER_USERNAME}"
MASTER_PASSWORD="${MASTER_PASSWORD}"

EOF

	return 0
}


################################################################################
# activate_replication function
function do_activate_replication()
{
	msg -d "START: do_activate_replication()."

	msg -i "CMD: ${PROGRAM_NAME} --activate-replication [with relevant options: --nodeid=${NODE_ID} --dbrelease=${DB_RELEASE} --datarelease=${DB_DATA_RELEASE} --master-node-name=${MASTER_NODE_NAME} --replica-node-name=${REPLICA_NODE_NAME} --master-connection-string=${MASTER_CONNECTION_STRING} --master-username=${MASTER_USERNAME} --master-password=<password>]"

	# Check needed input args
	arg_check "--nodeid" "${NODE_ID}"
	arg_check "--dbrelease" "${DB_RELEASE}"
	arg_check "--datarelease" "${DB_DATA_RELEASE}"
	arg_check "--master-node-name" "${MASTER_NODE_NAME}"
	arg_check "--replica-node-name" "${REPLICA_NODE_NAME}"
	arg_check "--master-connection-string" "${MASTER_CONNECTION_STRING}"
	arg_check "--master-username" "${MASTER_USERNAME}"
	arg_check "--master-password" "${MASTER_PASSWORD}"

	#############################################################################
	# Ask HAM for permission to activate replication
	msg -d "Ask HAM for permission to activate replication."
	mwd_send_msg_reply ${EV_ACT_REPL_PERM}
	if [ "x${MWDMSGS_REPLY}" != "x${EV_ACT_REPL_GR}" ]
	then
		LOG_MSG="'Activativate replication' not granted by HAM."
		msg -e "${LOG_MSG}"
		do_exit ${ERROR_OPER_NOT_GRANTED} "${LOG_MSG}"
	fi

	#############################################################################
	# Get node state.
	msg -d "Get node state."
	mwd_get_node_state

	#############################################################################
	# Node must be in ACTIVE state.
	if [ "x${NODE_STATE}" != "xEV_NODE_ST_ACT" ]
	then
		LOG_MSG="Node is not in ACTIVE state. Operation not permitted."
		msg -e "${LOG_MSG}"
		do_exit ${NEI_ERR} "${LOG_MSG}"
	fi

	db_activate_replication
	rv=$?

	if [ ${rv} -eq 0 ]
	then
		do_save_replica_info
	fi

	msg -d "END: do_activate_replication() [rv=${rv}]."
	return ${rv}
}


################################################################################
# deactivate_replication function
function do_deactivate_replication()
{
	msg -i "CMD: ${PROGRAM_NAME} --deactivate-replication [with relevant options:${msg_option_nodeid} --dbrelease=${DB_RELEASE} --datarelease=${DB_DATA_RELEASE} --master-node-name=${MASTER_NODE_NAME} --replica-node-name=${REPLICA_NODE_NAME} --master-connection-string=${MASTER_CONNECTION_STRING} --master-username=${MASTER_USERNAME} --master-password=<password>]"

	# Check needed input args
	arg_check "--dbrelease" "${DB_RELEASE}"
	arg_check "--datarelease" "${DB_DATA_RELEASE}"
	arg_check "--master-node-name" "${MASTER_NODE_NAME}"
	arg_check "--replica-node-name" "${REPLICA_NODE_NAME}"
	arg_check "--master-connection-string" "${MASTER_CONNECTION_STRING}"
	arg_check "--master-username" "${MASTER_USERNAME}"
	arg_check "--master-password" "${MASTER_PASSWORD}"

	if [ "x${NODE_ID}" = "x" ]
	then
		unset msg_option_nodeid
	else
		msg_option_nodeid=' --nodeid='${NODE_ID}
	fi

	#############################################################################
	# Ask HAM for permission to activate replication
	msg -d "Ask HAM for permission to activate replication."
	mwd_send_msg_reply ${EV_ACT_REPL_PERM}
	if [ "x${MWDMSGS_REPLY}" != "x${EV_ACT_REPL_GR}" ]
	then
		LOG_MSG="'Deactivativate replication' not granted by HAM."
		msg -e "${LOG_MSG}"
		do_exit ${ERROR_OPER_NOT_GRANTED} "${LOG_MSG}"
	fi

	#############################################################################
	# Get node state.
	msg -d "Get node state."
	mwd_get_node_state

	#############################################################################
	# Node must be in ACTIVE state.
	if [ "x${NODE_STATE}" != "xEV_NODE_ST_ACT" ]
	then
		LOG_MSG="Node is not in ACTIVE state. Operation not permitted."
		msg -e "${LOG_MSG}"
		do_exit ${NEI_ERR} "${LOG_MSG}"
	fi

	db_deactivate_replication
	rv=$?

	msg -d "END: do_deactivate_replication() [rv=${rv}]."
	return ${rv}
}


################################################################################
# refresh function
function do_refresh()
{
# ./element-admin.sh --refresh --nodeid=2010 --dbrelease=YGWW0C46 --datarelease=YGYC0A50 

	msg -i "CMD: ${PROGRAM_NAME} --refresh [with relevant options: --nodeid=${NODE_ID} --dbrelease=${DB_RELEASE} --datarelease=${DB_DATA_RELEASE}]"

	# Check needed input args
	arg_check "--nodeid" "${NODE_ID}"
	arg_check "--dbrelease" "${DB_RELEASE}"
	arg_check "--datarelease" "${DB_DATA_RELEASE}"

	#############################################################################
	# Get node state.
	msg -d "Get node state."
	mwd_get_node_state

	#############################################################################
	# If we are not upgrading SW, then node should be in ACTIVE state.   
  if [ "x${COMMAND_NAME}" != "x${UPGRADE_ARG}" ]
	then
		if [ "x${NODE_STATE}" != "xEV_NODE_ST_ACT" ]
		then
			LOG_MSG="Node is not in ACTIVE state. Operation not permitted."
			msg -e "${LOG_MSG}"
			do_exit ${NEI_ERR} "${LOG_MSG}"
		fi
	fi

	db_refresh
	rv=$?

	msg -d "END: do_refresh() [rv=${rv}]."
	return ${rv}
}


################################################################################
# Major DB upgrade function
function do_upgrade()
{
# ./element-admin.sh --upgrade --nodeid=2010 --dbrelease=YGWW0C46 --datarelease=YGYC0A50 --master-node-name=RHEL4EVA_IT_YGWW0C_YGYC0A --replica-node-name=2010 --master-connection-string="tcp rhel4eva.iskratel.si 1319" --master-username=itsolidmn --master-password=SGMvvYbN

	msg -d "START: do_upgrade()."

	msg -i "CMD: ${PROGRAM_NAME} --upgrade [with relevant options: --nodeid=${NODE_ID} --dbrelease=${DB_RELEASE} --datarelease=${DB_DATA_RELEASE} --master-node-name=${MASTER_NODE_NAME} --replica-node-name=${REPLICA_NODE_NAME} --master-connection-string=${MASTER_CONNECTION_STRING} --master-username=${MASTER_USERNAME} --master-password=<password>]"

	# Check needed input args
	arg_check "--nodeid" "${NODE_ID}"
	arg_check "--dbrelease" "${DB_RELEASE}"
	arg_check "--datarelease" "${DB_DATA_RELEASE}"
	arg_check "--master-node-name" "${MASTER_NODE_NAME}"
	arg_check "--replica-node-name" "${REPLICA_NODE_NAME}"
	arg_check "--master-connection-string" "${MASTER_CONNECTION_STRING}"
	arg_check "--master-username" "${MASTER_USERNAME}"
	arg_check "--master-password" "${MASTER_PASSWORD}"

	#############################################################################
	# Get node state.
	msg -d "Get node state."
	mwd_get_node_state

	# Define NO_EXIT so that functions below will not exit.
	NO_EXIT="Do not exit."
	UPGRADE_replication_activated=

	db_upgrade
	rv=$?

	msg -d "END: do_upgrade() [rv=${rv}]."
	return ${rv}
}


################################################################################
# Install data in DB from files on disc
# Required: file /etc/sysconfig/db-data with DB parameters
function do_install_data_from_disc()
{
	msg -d "START: do_install_data_from_disc()."

	msg -i "CMD: ${PROGRAM_NAME} --install-data-from-disc"

	#############################################################################
	# Check if input configuration file exists.
	if [ ! -e ${PRODUCT_DB_DATA_CFG} ]
	then
		LOG_MSG="System not configured properly. Missing file ${PRODUCT_DB_DATA_CFG}. Please, run /sbin/db-data."
		msg -e "${LOG_MSG}"
		do_exit ${NEI_ERR} "${LOG_MSG}"
	else
		. ${PRODUCT_DB_DATA_CFG}
	fi

	#############################################################################
	# Check if data should be loaded or skipped

	if [ "x${DB_DATA_LOAD}" = "xyes" -o "x${DB_DATA_LOAD}" = "x" ]
	then

		#############################################################################
		# Get node state.
		msg -d "Get node state."
		mwd_get_node_state

		#############################################################################
		# Load data as configured in ${PRODUCT_DB_DATA_CFG}
		db_install_data_from_disc --product=${PRODUCT}
		rv=$?

	else
		msg -d "Loading data is skipped by install command."
		rv=0
	fi

	msg -d "END: do_install_data_from_disc [rv=${rv}]."
	return ${rv}
}

################################################################################
################################################################################

# Main part
unset COMMAND_NAME

unset INSTALL_WITHOUT_LOAD_DB_DATA_CMD

# Define default DB server
export DB_SERVER_SYSTEM=${DB_SERVER_SYSTEM_DEFAULT}

# Show help and exit in case empty cmd switch
if [ -z "$1" ]
then
	show_help
	exit 1
fi

until [ -z "$1" ]
do
	case $1 in
		############################################################################
		# General functions
		############################################################################

		--help)
			if [ -z "${COMMAND_NAME}" ]
			then
				show_help
				exit 0
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--get-api-version)
			if [ -z "${COMMAND_NAME}" ]
			then
				echo "${PROGRAM_VERSION}"
				exit 0
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		############################################################################
		# System management commands
		############################################################################

		--sys-info)
			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_sys_info
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--list-pkg)
			if [ -z "${COMMAND_NAME}" ]
			then
				COMMAND_NAME=do_list_pkg
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--test-running-installation)
			do_check_if_installation_in_progress
			rv=$?
			exit ${rv}
			;;

		--install)
			# Check if installation on this node is not running
			do_check_if_installation_in_progress
			rv=$?

			if [ ${rv} -gt 0 ]
			then
				echo "Error: Instalation on this system is already running."
				echo
				exit 1
			fi

			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_install_software

				# Setup install environment
				echo "INSTALL_FILESYSTEM=yes" >> ${TMP_ENV}
				echo "INSTALL_SYSTEM_PLATFORM=yes" >> ${TMP_ENV}
				echo "INSTALL_PRODUCT=yes" >> ${TMP_ENV}
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--install-filesystem-only)
			# Check if installation on this node is not running
			do_check_if_installation_in_progress
			rv=$?
			
			if [ ${rv} -gt 0 ]
			then
				echo "Error: Instalation on this system is already running."
				echo
				exit 1
			fi

			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_install_software

				# Setup install environment
				echo "INSTALL_FILESYSTEM=yes" >> ${TMP_ENV}
				echo "INSTALL_SYSTEM_PLATFORM=no" >> ${TMP_ENV}
				echo "INSTALL_PRODUCT=no" >> ${TMP_ENV}
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--install-platform-only)
			# Check if installation on this node is not running
			do_check_if_installation_in_progress
			rv=$?
			
			if [ ${rv} -gt 0 ]
			then
				echo "Error: Instalation on this system is already running."
				echo
				exit 1
			fi

			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_install_software

				# Setup install environment
				echo "INSTALL_FILESYSTEM=yes" >> ${TMP_ENV}
				echo "INSTALL_SYSTEM_PLATFORM=yes" >> ${TMP_ENV}
				echo "INSTALL_PRODUCT=no" >> ${TMP_ENV}
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--install-on-working)
			# Check if installation on this node is not running
			do_check_if_installation_in_progress
			rv=$?
			
			if [ ${rv} -gt 0 ]
			then
				echo "Error: Instalation on this system is already running."
				echo
				exit 1
			fi

			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_install_software

				# Setup install environment
				echo "INSTALL_FILESYSTEM=no" >> ${TMP_ENV}
				echo "INSTALL_SYSTEM_PLATFORM=yes" >> ${TMP_ENV}
				echo "INSTALL_PRODUCT=yes" >> ${TMP_ENV}
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

			# set_active_release
		--set-active-release)
			if [ -z "${COMMAND_NAME}" ]
			then
				COMMAND_NAME=do_set_active_release
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		############################################################################
		# Database management commands
		############################################################################

		--get-solid-uid)
			if [ -z "${COMMAND_NAME}" ]
			then
				COMMAND_NAME=do_get_solid_uid
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--show-db-status)
			if [ -z "${COMMAND_NAME}" ]
			then
				COMMAND_NAME=db_server_and_data_status
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--activate-replication)
			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_activate_replication
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--deactivate-replication)
			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_deactivate_replication
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--refresh)
			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_refresh
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--upgrade)
			if [ -z "${COMMAND_NAME}" ]
			then
				COMMAND_NAME=do_upgrade
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--install-data-from-disc)
			if [ -z "${COMMAND_NAME}" ]
			then
				disable_sysup_actions
				COMMAND_NAME=do_install_data_from_disc
			else
				echo "Error: only one command is allowed."
				echo
				show_help
				exit 1
			fi
			;;

		--without-db-data)
			# Setup install environment
			echo "INSTALL_WITHOUT_DB_DATA=yes" >> ${TMP_ENV}
			;;

		--without-load-db-data)
			# Setup install environment
			echo "INSTALL_WITHOUT_LOAD_DB_DATA=yes" >> ${TMP_ENV}
			;;


		############################################################################
		# Public arguments part
		############################################################################

		--use-api-version*)
			export USE_API_VERSION=$(echo ${1/--use-api-version=/})

			# security check of input argument
			input_args_security_check "--use-api-version" "${USE_API_VERSION}"
			;;

		--repo-sw-url*)
			export REPO_SW_URL=$(echo ${1/--repo-sw-url=/})

			# security check of input argument
			input_args_security_check "--repo-sw-url" "${REPO_SW_URL}"

			# Setup install environment
			echo "REPO_SW_URL=${REPO_SW_URL}" >> ${TMP_ENV}
			;;

		--repo-fs-url*)
			export REPO_FS_URL=$(echo ${1/--repo-fs-url=/})

			# security check of input argument
			input_args_security_check "--repo-fs-url" "${REPO_FS_URL}"

			# Setup install environment
			echo "REPO_FS_URL=${REPO_FS_URL}" >> ${TMP_ENV}
			;;

		--target-arch*)
			TARGET_ARCH=$(echo ${1/--target-arch=/})

			# security check of input argument
			input_args_security_check "--target-arch" "${TARGET_ARCH}"

			# Setup install environment
			echo "TARGET_ARCH=${TARGET_ARCH}" >> ${TMP_ENV}
			;;

		--target-board-name*)
			export TARGET_BOARD_NAME=$(echo ${1/--target-board-name=/})

			# security check of input argument
			input_args_security_check "--target-board-name" "${TARGET_BOARD_NAME}"

			export TARGET_BOARD_NAME_OPTS="--target-board-name=${TARGET_BOARD_NAME}"

			# Setup install environment
			echo "TARGET_BOARD_NAME=${TARGET_BOARD_NAME}" >> ${TMP_ENV}
			;;

		--nodeid*)
			export NODE_ID=$(echo ${1/--nodeid=/})

			# security check of input argument
			input_args_security_check "--nodeid" "${NODE_ID}"

			# Setup install environment
			echo "NODE_ID=${NODE_ID}" >> ${TMP_ENV}
			;;

		--db-server*)
			# Define DB server to change default from value DB_SERVER_SYSTEM_DEFAULT
			export DB_SERVER_SYSTEM=$(echo ${1/--db-server=/})

			# security check of input argument
			input_args_security_check "--db-server" "${DB_SERVER_SYSTEM}"

			case ${DB_SERVER_SYSTEM} in
				postgresql|netezza|solid)

					# Setup install environment
					echo "DB_SERVER_SYSTEM=${DB_SERVER_SYSTEM}" >> ${TMP_ENV}
					;;
				*)
					echo "Error: unknown DB server [${DB_SERVER_SYSTEM}]"
					show_help
					do_exit 1
					;;
			esac
			;;

		--dbrelease*)
			export DB_RELEASE=$(echo ${1/--dbrelease=/})

			# security check of input argument
			input_args_security_check "--dbrelease" "${DB_RELEASE}"

			# Setup install environment
			echo "DB_RELEASE=${DB_RELEASE}" >> ${TMP_ENV}
			;;

		--datarelease*)
			export DB_DATA_RELEASE=$(echo ${1/--datarelease=/})

			# security check of input argument
			input_args_security_check "--datarelease" "${DB_DATA_RELEASE}"

			# Setup install environment
			echo "DB_DATA_RELEASE=${DB_DATA_RELEASE}" >> ${TMP_ENV}
			;;

		--data-type*)
			export DATA_RELEASE_TYPE=$(echo ${1/--data-type=/})

			# security check of input argument
			input_args_security_check "--data-type" "${DATA_RELEASE_TYPE}"

			# Setup install environment
			echo "DATA_RELEASE_TYPE=${DATA_RELEASE_TYPE}" >> ${TMP_ENV}
			;;

		--catalog-name*)
			export DB_CATALOG_NAME=$(echo ${1/--catalog-name=/})

			# security check of input argument
			input_args_security_check "--catalog-name" "${DB_CATALOG_NAME}"

			# Setup install environment
			echo "DB_CATALOG_NAME=${DB_CATALOG_NAME}" >> ${TMP_ENV}
			;;

		--housing*)
			export DB_HOUSING=$(echo ${1/--housing=/})

			# security check of input argument
			input_args_security_check "--housing" "${DB_HOUSING}"

			# Setup install environment
			echo "DB_HOUSING=${DB_HOUSING}" >> ${TMP_ENV}
			;;

		--master-node-name*)
			export MASTER_NODE_NAME=$(echo ${1/--master-node-name=/})

			# security check of input argument
			input_args_security_check "--master-node-name" "${MASTER_NODE_NAME}"
			;;

		--replica-node-name*)
			export REPLICA_NODE_NAME=$(echo ${1/--replica-node-name=/})

			# security check of input argument
			input_args_security_check "--replica-node-name" "${REPLICA_NODE_NAME}"
			;;

		--master-connection-string*)
			export MASTER_CONNECTION_STRING=$(echo ${1/--master-connection-string=/})
			;;

		--master-username*)
			export MASTER_USERNAME=$(echo ${1/--master-username=/})

			# security check of input argument
			input_args_security_check "--master-username" "${MASTER_USERNAME}"
			;;

		--master-password*)
			export MASTER_PASSWORD=$(echo ${1/--master-password=/})

			# security check of input argument
			input_args_security_check "--master-password" "${MASTER_PASSWORD}"
			;;

		--product*)
			export PRODUCT=$(echo ${1/--product=/})

			# security check of input argument
			input_args_security_check "--product" "${PRODUCT}"

			# Setup install environment
			echo "PRODUCT=${PRODUCT}" >> ${TMP_ENV}
			;;

		--no-ssh)
			export COMMAND_VIA_SSH="no"
			;;

		*)
			echo
			echo "Error: Unknown argument $1"
			show_help
			do_exit 1
			;;
	esac
	shift
done

HOST_STRING=""
if [ "x${COMMAND_VIA_SSH}" != "xno" ]; then
	is_sshd_running=$(pidof sshd &>/dev/null; echo $?)
	if [ $is_sshd_running -eq 0 ]; then
		HOST_STRING="localhost"
	fi
fi

${COMMAND_NAME}
do_exit $?
